﻿using Modbus.Messages;

namespace Modbus
{
	partial class ModbusClient
	{
		public void MaskWriteRegister(byte slaveAddress, int registerAddress, short andMask, short orMask)
		{
			//
			// Input arguments validation.
			//

			Ensure.ArgumentInRange(SR.Argument_registerAddress, registerAddress, MinDataAddress, MaxDataAddress);

			//
			// Internal state validation.
			//

			this.ThrowIfDisposed();

			//
			// Request creation.
			//

			MaskWriteRegisterMessage request = new MaskWriteRegisterMessage();

			request.FunctionCode = FunctionCodes.MaskWriteRegister;
			request.RegisterAddress = (ushort)(registerAddress);
			request.AndMask = andMask;
			request.OrMask = orMask;

			//
			// Perform transaction.
			//
			
			if (slaveAddress == BroadcastAddress)
			{
				this.transport.Request(slaveAddress, request);
			}
			else
			{
				var response = this.transport.Request<MaskWriteRegisterMessage>(slaveAddress, request);
				
				//
				// Response validation.
				//

				if (response.RegisterAddress != request.RegisterAddress)
				{
					throw new MismatchResponseException(SR.ModbusMessageField_RegisterAddress);
				}
				if (response.AndMask != request.AndMask)
				{
					throw new MismatchResponseException(SR.ModbusMessageField_AndMask);
				}
				if (response.OrMask != request.OrMask)
				{
					throw new MismatchResponseException(SR.ModbusMessageField_OrMask);
				}
			}
		}
	}
}
