﻿using Modbus.Messages;

namespace Modbus
{
	partial class ModbusClient
	{
		public void WriteMultipleRegisters(byte slaveAddress, int startingAddress, short[] registers)
		{
			//
			// Input arguments validation.
			//

			Ensure.ArgumentInRange(SR.Argument_startingAddress, startingAddress, MinDataAddress, MaxDataAddress);
			Ensure.ArrayOfRequiredLength(SR.Argument_registers, registers, ModbusSpecification.MinQuantity,
				ModbusSpecification.WriteMultipleRegistersMaxQuantity);

			//
			// Internal state validation.
			//

			this.ThrowIfDisposed();

			//
			// Request creation.
			//

			WriteMultipleRegistersRequest request = new WriteMultipleRegistersRequest();

			request.FunctionCode = FunctionCodes.WriteMultipleRegisters;
			request.StartingAddress = (ushort)(startingAddress);
			request.Quantity = (ushort)(registers.Length);
			request.Data = DataUtility.EncodeRegisters(registers);

			//
			// Perform transaction.
			//

			if (slaveAddress == BroadcastAddress)
			{
				this.transport.Request(slaveAddress, request);
			}
			else
			{
				var response = this.transport.Request<StartingAddressQuantityMessage>(slaveAddress, request);

				//
				// Response validation.
				//

				if (response.StartingAddress != request.StartingAddress)
				{
					throw new MismatchResponseException(SR.ModbusMessageField_StartingAddress);
				}
				if (response.Quantity != request.Quantity)
				{
					throw new MismatchResponseException(SR.ModbusMessageField_Quantity);
				}
			}
		}
	}
}
